﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

using BaseDockObjects;

namespace CircleDock
{
    partial class DockItemProperties : Form
    {
        #region Variables

        DockItemObject ParentObject;
        LanguageLoader.LanguageLoader Language;
        SettingsLoader.SettingsLoader DockSettings;
        DockItemSettingsLoader.DockItemSettingsLoader DockItemSettings;
        bool Loaded = false;

        #endregion

        public DockItemProperties(DockItemObject TheParent, LanguageLoader.LanguageLoader LanguageData, SettingsLoader.SettingsLoader SettingsData,
            DockItemSettingsLoader.DockItemSettingsLoader DockItemSettingsData, String SectionName)
        {
            ParentObject = TheParent;
            Language = LanguageData;
            DockSettings = SettingsData;
            DockItemSettings = DockItemSettingsData;
            Loaded = false;

            InitializeComponent();
            InitializeSettings();
        }

        public void InitializeSettings()
        {
            #region Title / Banner

            this.Text = Language.DockItemProperties.ItemSettings;

            #endregion

            #region Icon Image

            IconImageLabel.Text = Language.DockItemProperties.IconImage;
            IconImageFileDialog.Title = Language.DockItemProperties.IconImage;
            IconImagePictureBox.Image = new Bitmap(ParentObject.ObjectBitmap);

            String ImagePath = @DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "ImagePath");

            if (ImagePath != null)
            {
                IconImageTextBox.Text = ImagePath;
            }
            else
            {
                IconImageTextBox.Text = @"";
            }

            #endregion

            #region Name

            NameLabel.Text = Language.DockItemProperties.Name;
            String tempName = DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "Name");

            if (tempName != null && tempName.Length > 0)
            {
                NameTextBox.Text = tempName;
            }
            else
            {
                NameTextBox.Text = ParentObject.FileOperations.Name;
            }

            #endregion

            #region Target

            TargetLabel.Text = Language.DockItemProperties.Target;
            TargetOpenFileDialog.Title = Language.DockItemProperties.Target;
            TargetFolderBrowserDialog.Description = Language.DockItemProperties.Target;
            if (DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "Action") == @"[dockfolder]")
            {
                TargetTextBox.Text = ParentObject.DockItemSectionName + @"-";
                TargetTextBox.ReadOnly = true;
                TargetFileBrowse.Enabled = false;
                TargetFolderBrowse.Enabled = false;
            }
            else
            {
                TargetTextBox.Text = DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "Args");
            }

            TargetFileBrowse.Text = Language.DockItemProperties.File;
            TargetFolderBrowse.Text = Language.DockItemProperties.Folder;

            #endregion

            #region Start In

            StartInLabel.Text = Language.DockItemProperties.StartIn;

            #endregion

            #region Arguments

            ArgumentsLabel.Text = Language.DockItemProperties.Arguments;

            String argsParams = DockItemSettings.GetEntry(ParentObject.DockItemSectionName, @"ArgsParams");

            if (argsParams == null)
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"ArgsParams", "");
                ArgumentsTextBox.Text = @"";
            }
            else
            {
                ArgumentsTextBox.Text = argsParams;
            }

            #endregion

            #region Run

            RunLabel.Text = Language.DockItemProperties.Run;
            RunComboBox.Items.Clear();
            RunComboBox.Items.Add(Language.DockItemProperties.Normal);
            RunComboBox.Items.Add(Language.DockItemProperties.Minimized);
            RunComboBox.Items.Add(Language.DockItemProperties.Maximized);

            String runAsArg = DockItemSettings.GetEntry(ParentObject.DockItemSectionName, @"RunAs");

            if (runAsArg == null || (runAsArg != @"Normal" && runAsArg != "Minimized" && runAsArg != "Maximized"))
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"RunAs", @"Normal");
                RunComboBox.SelectedIndex = 0;
            }
            else
            {
                if (runAsArg == @"Maximized")
                    RunComboBox.SelectedIndex = 2;
                else if (runAsArg == @"Minimized")
                    RunComboBox.SelectedIndex = 1;
                else
                    RunComboBox.SelectedIndex = 0;
            }

            #endregion

            #region Close

            closeButton.Text = Language.DockItemProperties.Close;

            #endregion

            Loaded = true;
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            //
            // If stack object we must ensure that the command-line arguement includes the
            // name of the stack so as to allow more than one stack object on the dock. The
            // "/notaskbar" argument prevents the stack object creating an (unwanted) taskbar
            // entry.
            //
            string s = TargetTextBox.Text.ToLower();

            if (s == ".\\system\\miscellaneous\\standalonestack.exe")
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"ArgsParams", "\"" + NameTextBox.Text + "\"" + " /notaskbar /right");
            }
            //
            //
            this.Select(true, true);
            this.Close();
        }

        #region Icon Image

        private void IconImageTextBox_Leave(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            if (IconImageTextBox.Text == DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "ImagePath"))
                return;
            
            String NewImagePath;
            if (IconImageTextBox.Text.StartsWith(Application.StartupPath + @"\"))
            {
                NewImagePath = @"." + IconImageTextBox.Text.Substring(Application.StartupPath.Length, IconImageTextBox.Text.Length - Application.StartupPath.Length);
            }
            else
            {
                NewImagePath = IconImageTextBox.Text;
            }

            if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, "ImagePath", NewImagePath);
            }

            ParentObject.SetBitmapRedraw();
            IconImagePictureBox.Image = new Bitmap(ParentObject.ObjectBitmap);
        }

        private void IconImageTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Loaded == false)
                return;

            if (e.KeyChar == (char)Keys.Enter)
                this.Select(true, true);
        }

        private void IconImageBrowseButton_Click(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            IconImageFileDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            IconImageFileDialog.ShowDialog();
        }

        private void IconImageFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            if (Loaded == false)
                return;

            String NewImagePath;
            if (IconImageFileDialog.FileName.StartsWith(Application.StartupPath + @"\"))
            {
                NewImagePath = @"." + IconImageFileDialog.FileName.Substring(Application.StartupPath.Length, IconImageFileDialog.FileName.Length - Application.StartupPath.Length);
            }
            else
            {
                NewImagePath = IconImageFileDialog.FileName;
            }

            IconImageTextBox.Text = NewImagePath;

            if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, "ImagePath", NewImagePath);
            }

            ParentObject.SetBitmapRedraw();
            IconImagePictureBox.Image = new Bitmap(ParentObject.ObjectBitmap);
        }

        #endregion

        #region Target

        private void TargetFolderBrowse_Click(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            if (TargetFolderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                String NewFilePath;
                if (TargetFolderBrowserDialog.SelectedPath.StartsWith(Application.StartupPath + @"\"))
                {
                    NewFilePath = @"." + TargetFolderBrowserDialog.SelectedPath.Substring(Application.StartupPath.Length, TargetFolderBrowserDialog.SelectedPath.Length - Application.StartupPath.Length);
                    if (NewFilePath == @".")
                        NewFilePath = @".\";
                }
                else
                {
                    NewFilePath = TargetFolderBrowserDialog.SelectedPath;
                }

                TargetTextBox.Text = NewFilePath;

                if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
                {
                    DockItemSettings.SetEntry(ParentObject.DockItemSectionName, "Args", NewFilePath);
                }

                ParentObject.InitializeFileOps();
                ParentObject.SetBitmapRedraw();
                Loaded = false;
                InitializeSettings();
            }
        }

        private void TargetFileBrowse_Click(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            TargetOpenFileDialog.ShowDialog();
        }

        private void TargetOpenFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            if (Loaded == false)
                return;

            String NewFilePath;
            if (TargetOpenFileDialog.FileName.StartsWith(Application.StartupPath + @"\"))
            {
                NewFilePath = @"." + TargetOpenFileDialog.FileName.Substring(Application.StartupPath.Length, TargetOpenFileDialog.FileName.Length - Application.StartupPath.Length);
                if (NewFilePath == @".")
                    NewFilePath = @".\";
            }
            else
            {
                NewFilePath = TargetOpenFileDialog.FileName;
            }

            TargetTextBox.Text = NewFilePath;

            if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, "Args", NewFilePath);
            }

            ParentObject.InitializeFileOps();
            ParentObject.SetBitmapRedraw();
            Loaded = false;
            InitializeSettings();
        }

        private void TargetTextBox_Leave(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            if (TargetTextBox.ReadOnly == true)
                return;

            if (TargetTextBox.Text == DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "Args"))
                return;

            if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
            {
                String NewFilePath = TargetTextBox.Text;
                if (TargetTextBox.Text.StartsWith(Application.StartupPath + @"\"))
                {
                    NewFilePath = @"." + TargetTextBox.Text.Substring(Application.StartupPath.Length, TargetTextBox.Text.Length - Application.StartupPath.Length);
                    if (NewFilePath == @".")
                        NewFilePath = @".\";
                }
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, "Args", NewFilePath);
            }

            ParentObject.InitializeFileOps();
            ParentObject.SetBitmapRedraw();
            Loaded = false;
            InitializeSettings();
        }

        private void TargetTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Loaded == false)
                return;

            if (e.KeyChar == (char)Keys.Enter)
                this.Select(true, true);
        }

        #endregion

        private void NameTextBox_Leave(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            if (NameTextBox.Text == DockItemSettings.GetEntry(ParentObject.DockItemSectionName, "Name"))
                return;

            if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, "Name", NameTextBox.Text);
            }

            ParentObject.InitializeFileOps();

            if (NameTextBox.Text.Trim().Length == 0)
                NameTextBox.Text = ParentObject.FileOperations.Name;
        }

        private void NameTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Loaded == false)
                return;

            if (e.KeyChar == (char)Keys.Enter)
            {
                if (NameTextBox.Text.Trim().Length == 0)
                    NameTextBox.Text = ParentObject.FileOperations.Name;
                this.Select(true, true);
            }
        }



        private void RunComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            if (RunComboBox.SelectedIndex == 2)
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"RunAs", @"Maximized");
            else if (RunComboBox.SelectedIndex == 1)
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"RunAs", @"Minimized");
            else
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"RunAs", @"Normal");
        }

        private void ArgumentsTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Loaded == false)
                return;

            if (e.KeyChar == (char)Keys.Enter)
            {
                this.Select(true, true);
            }
        }

        private void ArgumentsTextBox_Leave(object sender, EventArgs e)
        {
            if (Loaded == false)
                return;

            if (ParentObject.DockItemSectionName != null && ParentObject.DockItemSectionName != "")
            {
                DockItemSettings.SetEntry(ParentObject.DockItemSectionName, @"ArgsParams", ArgumentsTextBox.Text);
            }
        }

    }
}
