﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// This file contains the class used to load the .ini language file from a given path.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;

namespace LanguageInformation
{
    public struct LanguageFile
    {
        public String CircleDockVersion;
        public String LanguageFileVersion;
        public String LanguageName;
        public String FileAuthor;
    }

    public struct General
    {
        public String CircleDockName;
        public String CircleDockWebsiteWord;
        public String GeneralWebsite;
        public String HelpWebsite;
        public String AuthorWord;
        public String SupportForumWord;
        public String SupportForumWebsite;
        public String RotationSensitivity;
        public String IconReplacementMode;

        public String normal;
        public String topmost;
        public String alwaysOnBottom;
        public String zLevel;
    }

    public struct MainContextMenu
    {
        public String WebsiteWord;
        public String SettingsWord;
        public String HideWord;
        public String ShowWord;
        public String QuitWord;
        public String RemoveWord;
        public String AddWord;
        public String DockFolder;
        public String ChangeIcon;
        public String ItemSettings;
        public String PauseMouseToggling;
        public String blankIcon;
    }

    public struct SettingsPanel
    {
        public String Name;
        public String SettingsPanelWord;
        public String General;
        
        public String DockShape;
        
        public String Labels;
        public String Animation;
        public String LocationWord;
        public String TogglingWord;
        public String FileDockIconAssociations;
        public String LanguageWord;
        public String HelpWord;
        public String CircleDockVersionWord;
        public String EnableDockRotation;
        public String HotKeySelectorToggleDock;
        public String CtrlWordAbbreviation;
        public String AltWordAbbreviation;
        public String ShiftWordAbbreviation;
        public String WindowsKeyWordAbbreviation;
        public String ValidHotkeyCombination;
        public String InvalidHotKeyCombination;

        // Location Panel
        public String CentreAroundCursorWhenShown;
        public String lockDockAtCurrentPosition;

        // General Panel
        public String ConfigurationDescriptionLabel;
        public String KeepDockAboveAllOtherWindows;
        public String ClickSensitivity;
        public String Sensitive;
        public String NotSensitive;
        public String EnablePortabilityMode;
        public String UseMemorySaver;
        public String useSameRotationValueAllLevels;
        public String currentRotationValue;

        // Background Panel
        public String Background;
        public String BackgroundImage;
        public String NormalSize;
        public String NormalOpacity;

        // Dock Shape Panel
        public String ConstantNumberOfItemsPerCircle;
        public String NumberOfIconsPerCircle;
        public String MinimumRadius;
        public String SeparationBetweenCircles;
        public String FormatWord;
        public String CircleWord;
        public String maximumNumberOfItemsPerCircle;
        public String lineWord;
        public String spiralWord;
        public String rateOfIncrease;
        public String defaultWord;
        public String separationWord;


        // Centre Button Panel
        public String CentreButton;
        public String DefaultCentreButton;
        public String OtherSettings;
        public String ShowStartMenuWhenClicked;
        public String StartMenuResidueWarning;

        // Dock Items Panel
        public String DockItems;
        public String RightClickMenu;
        public String ShowWindowsFileFolderMenus;
        public String HideDock;
        public String HideDockAfterFileExecution;

        // Labels Panel
        public String FontWord;
        public String ChangeFont;
        public String ChangeColor;
        public String ShadowWord;
        public String ShadowSize;
        public String ShadowDarknessFactor;
        public String ShowLabels;
        public String ShowLabelsOnMouseoverOnly;

        // Animation Panel
        public String AnimationWord;
        public String GeneralFrameRate;
        public String RotationAnimationDuration;
        public String FadeInFadeOutDuration;
        public String UsePoofAnimationWhenDeleting;

        // About Panel
        public String AboutWord;
        public String ThePremiereCircularDockForWindows;
        public String WebsiteWord;
        public String VersionInformation;
        public String OfficialHomepage;

        // Help Panel
        public String SupportWord;
        public String OfficalWebForum;
        public String OfficialWebHelpFAQ;

        // Language Panel
        public String LanguageFileLocation;
        public String LanguageName;
        public String LanguageFileAuthor;
        public String LanguageFileVersion;
        public String LanguageFileOriginallyIntendedFor;
        public String LanguageFileProgramRestartRequired;
        public String LanguageFileValidCurrent;
        public String LanguageFileInvalidOutofDate;

        // File -> Icon Associations Panel
        public String FileIconAssocationsDescription;
        public String FileNameIconAssociations;
        public String FolderNameIconAssociations;
        public String FileExtensionIconAssociations;
        public String DockFolderIconAssociation;

        public String fileName;
        public String folderName;
        public String extensionName;
        public String associatedDockIcon;
        public String browse;
        public String startsWith;
        public String endsWith;
        public String exactMatch;
        public String contains;
        public String sample;

        // Toggling
        public String ToggleVisibilityHotKey;
        public String ToggleVisibilityMouseButton;
        public String ShowDockWhenIMoveMyMouseTo;
        public String ScreenLeftEdge;
        public String ScreenRightEdge;
        public String ScreenTopEdge;
        public String ScreenBottomEdge;
        public String EdgeWidthHeight;
        public String dwellTime;
    }

    public struct ErrorMessages
    {
        public String MissingInvalid;
        public String FilePathMissingInvalid;
    }

    public struct DockItemProperties
    {
        public String ItemSettings;
        public String IconImage;
        public String Name;
        public String Target;
        public String Folder;
        public String File;
        public String StartIn;
        public String Arguments;
        public String Run;
        public String Normal;
        public String Minimized;
        public String Maximized;
        public String Close;
    }
}
