﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// This file contains the structs used to hold the settings information for the dock.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;

/// <summary>
/// The namespace SettingsInformation is a collection of structs that is used to hold the general dock settings for Circle Dock.
/// It is used by SettingsLoader to load the general settings from an Config.ini file and allow the program to access the data.
/// The settings are loaded from the Config.ini file into memory to allow for faster performance.
/// Individual dock item settings are dealt with DockItemsInformation and DockItemSettingsLoader.
/// </summary>
namespace SettingsInformation
{
    public struct Background
    {
        /// <summary>
        /// File path to the image for the background.
        /// </summary>
        public String Path;

        /// <summary>
        /// Width of the background image in pixels.
        /// </summary>
        public int DefaultWidth;

        /// <summary>
        /// Height of the background image in pixels.
        /// </summary>
        public int DefaultHeight;

        /// <summary>
        /// Opacity of the background image in pixels.
        /// </summary>
        public int DefaultOpacity;
    }

    public struct CentreImage
    {
        /// <summary>
        /// File path to the default image for the centre image.
        /// </summary>
        public String Path;

        /// <summary>
        /// Width of the centre image in pixels.
        /// </summary>
        public int DefaultWidth;

        /// <summary>
        /// Height of the centre image in pixels.
        /// </summary>
        public int DefaultHeight;

        public int DefaultOpacity;

        public bool ShowStartMenuWhenClicked;
    }

    public struct DockItems
    {
        public int DefaultWidth;
        public int DefaultHeight;
        public int MinWidth;
        public int MinHeight;
        public int MaxWidth;
        public int MaxHeight;
        public bool ShowExplorerContextMenus;
        public bool HideDockAfterExecution;
        public int DefaultOpacity;
    }

    public struct CircleParams
    {
        public int MinRadius;
        public int CircleSeparation;
        public String Format;
        public int ConstNumItemsPerCircle;
    }

    public struct EllipseParams
    {
        public int MinHeight;
        public double AspectRatio;
    }

    public struct Language
    {
        public String path;
    }

    public struct ObjectShape
    {
        public String Shape;
    }

    public struct Folders
    {
        public String DockFolderImagePath;
    }

    public struct General
    {
        public bool ShowLevelRealTimeCreation;
        public bool CentreAroundCursor;
        public bool EnableDockRotation;
        public int AnimationInterval;
        public int ClickSensitivity;
        public bool EnablePortabilityMode;
        public int KeyPressesPerRevolution;
        public bool UseMemorySaver;
        public bool IconReplacementMode;
        public String zLevel;
        public bool lockDockPosition;
        public Point lockedPosition;
        public bool useSameRotationValues;
        public bool ShowDock;
        public bool AlwaysShowTopLevel;
    }

    public struct PoofAnimation
    {
        public String PoofAnimationPath;
        public int NumFrames;
        public int DelayBetweenFrames;
        public bool UsePoof;
    }

    public struct GeneralAnimation
    {
        public int RotationAnimationDuration;
        public int FadeInFadeOutDuration;
    }

    public struct Toggling
    {
        public bool VisibilityCtrl;
        public bool VisibilityAlt;
        public bool VisibilityShift;
        public bool VisibilityWin;
        public int VisibilityKey1;
        public int VisibilityKey2;

        public String VisibilityMouseButton;

        public bool ScreenLeftEdgeShow;
        public bool ScreenRightEdgeShow;
        public bool ScreenTopEdgeShow;
        public bool ScreenBottomEdgeShow;

        public int EdgeWidthHeight;
        public int dwellTime;

        public bool AutoHide;
    }

    public struct Labels
    {
        public Font FontName;
        public Color FontColor;
        public Color ShadowColor;
        public int ShadowSize;
        public int ShadowDarkness;
        public bool ShowLabels;
        public bool ShowLabelsOnMouseOverOnly;
        public int DefaultOpacity;
    }   
}
