﻿// Circle Dock 0.9.2 - Copyright 2008 Eric Wong
// Circle Dock is open source software licensed under GNU GENERAL PUBLIC LICENSE V3. See included Licence.txt file.
// http://circledock.wikidot.com
// VideoInPicture@gmail.com

// This file the objects used to build the part of Circle Dock that the user sees and interacts with.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;
using BaseDockObjects;

using CircleDock;
using Constants;
using DockItemsInformation;
using PerPixelAlphaForms;
using LanguageLoader;
using SettingsLoader;
using DockItemSettingsLoader;

using Orbit.Utilities;
using Win32.Shell32;
using FileOps;

using ShellExtension;
using System.IO;

namespace BaseDockObjects
{
    /// <para>Our test form for this sample application.  The bitmap will be displayed in this window.</para>
    //public partial class TransparentObject : Form
    public partial class BackgroundObject : PerPixelAlphaForms.BackgroundPerPixelAlphaForm
    {
        #region Windows Form Designer

        private ContextMenuStrip contextMenuStrip;
        private System.ComponentModel.IContainer components;
        private ToolStripMenuItem quitToolStripMenuItem;
        private ToolStripMenuItem addToolStripMenuItem;
        private ToolStripMenuItem hideToolStripMenuItem;
        private ToolStripMenuItem settingsToolStripMenuItem;
        private ToolStripMenuItem dockFolderToolStripMenuItem;
        private ToolStripMenuItem stackFolderToolStripMenuItem;
        private ToolStripMenuItem changeIconToolStripMenuItem;
        private ToolStripMenuItem iconReplacementModeToolStripMenuItem;
        private ToolStripMenuItem pauseMouseHotkeysToolStripMenuItem;
        private ToolStripMenuItem blankIconToolStripMenuItem;
        private ToolStripMenuItem toggleAutoHide;
        private OpenFileDialog ChangeIconDialog;
        public System.Windows.Forms.Timer dwellTimer;


        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.contextMenuStrip = new System.Windows.Forms.ContextMenuStrip(this.components);
            this.quitToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.addToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.hideToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.dockFolderToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.stackFolderToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.blankIconToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.settingsToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.changeIconToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.iconReplacementModeToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.pauseMouseHotkeysToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.toggleAutoHide = new System.Windows.Forms.ToolStripMenuItem();
            this.ChangeIconDialog = new System.Windows.Forms.OpenFileDialog();

            this.contextMenuStrip.SuspendLayout();
            this.SuspendLayout();
            // 
            // contextMenuStrip
            // 
            this.contextMenuStrip.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.hideToolStripMenuItem, this.addToolStripMenuItem, this.toggleAutoHide, this.changeIconToolStripMenuItem, this.iconReplacementModeToolStripMenuItem, 
                this.pauseMouseHotkeysToolStripMenuItem, this.settingsToolStripMenuItem, this.quitToolStripMenuItem});
            this.contextMenuStrip.Name = "contextMenuStrip";
            this.contextMenuStrip.Size = new System.Drawing.Size(153, 48);
            this.contextMenuStrip.Opening += new System.ComponentModel.CancelEventHandler(this.contextMenuStrip_Opening);
            this.contextMenuStrip.MouseEnter += new System.EventHandler(this.contextMenuStrip_MouseEnter);
            // 
            // quitToolStripMenuItem
            // 
            this.quitToolStripMenuItem.Name = "quitToolStripMenuItem";
            this.quitToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.quitToolStripMenuItem.Text = Language.MainContextMenu.QuitWord;
            this.quitToolStripMenuItem.Click += new System.EventHandler(this.quitToolStripMenuItem_Click);
            // 
            // addToolStripMenuItem
            // 
            this.addToolStripMenuItem.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.dockFolderToolStripMenuItem, this.blankIconToolStripMenuItem, stackFolderToolStripMenuItem});
            this.addToolStripMenuItem.Name = "addToolStripMenuItem";
            this.addToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.addToolStripMenuItem.Text = Language.MainContextMenu.AddWord;
            //
            // Toggle AutoHide
            //
            this.toggleAutoHide.Name = "toggleAutoHide";
            this.toggleAutoHide.Size = new System.Drawing.Size(152, 22);
            this.toggleAutoHide.Text = "AutoHide";
            this.toggleAutoHide.CheckOnClick = true;
            this.toggleAutoHide.Checked = DockSettings.Toggling.AutoHide;
            this.toggleAutoHide.Click += new System.EventHandler(this.toggleAutoHide_Click);
            // 
            // hideToolStripMenuItem
            // 
            this.hideToolStripMenuItem.Name = "hideToolStripMenuItem";
            this.hideToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.hideToolStripMenuItem.Text = Language.MainContextMenu.HideWord;
            this.hideToolStripMenuItem.Click += new System.EventHandler(this.hideToolStripMenuItem_Click);
            // 
            // settingsToolStripMenuItem
            // 
            this.settingsToolStripMenuItem.Name = "settingsToolStripMenuItem";
            this.settingsToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.settingsToolStripMenuItem.Text = Language.MainContextMenu.SettingsWord;
            this.settingsToolStripMenuItem.Click += new System.EventHandler(this.settingsToolStripMenuItem_Click);
            // 
            // iconReplacementModeToolStripMenuItem
            // 
            this.iconReplacementModeToolStripMenuItem.Name = "iconReplacementModeToolStripMenuItem";
            this.iconReplacementModeToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.iconReplacementModeToolStripMenuItem.Text = Language.General.IconReplacementMode;
            this.iconReplacementModeToolStripMenuItem.CheckOnClick = true;
            this.iconReplacementModeToolStripMenuItem.Checked = DockSettings.General.IconReplacementMode;
            this.iconReplacementModeToolStripMenuItem.Click += new System.EventHandler(this.iconReplacementModeToolStripMenuItem_Click);
            // 
            // pauseMouseHotkeysToolStripMenuItem
            // 
            this.pauseMouseHotkeysToolStripMenuItem.Name = "pauseMouseHotkeysToolStripMenuItem";
            this.pauseMouseHotkeysToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.pauseMouseHotkeysToolStripMenuItem.Text = Language.MainContextMenu.PauseMouseToggling;
            this.pauseMouseHotkeysToolStripMenuItem.CheckOnClick = true;
            this.pauseMouseHotkeysToolStripMenuItem.Click += new System.EventHandler(this.pauseMouseHotkeysToolStripMenuItem_Click);
            // 
            // OpenDataPointsDialog
            // 
            this.ChangeIconDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            this.ChangeIconDialog.Filter = "PNG|*.png";
            this.ChangeIconDialog.Multiselect = false;
            this.ChangeIconDialog.Title = Language.MainContextMenu.ChangeIcon;
            this.ChangeIconDialog.FileOk += new System.ComponentModel.CancelEventHandler(this.ChangeIconDialog_FileOk);
            // 
            // changeIconStripMenuItem
            // 
            this.changeIconToolStripMenuItem.Name = "changeIconToolStripMenuItem";
            this.changeIconToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.changeIconToolStripMenuItem.Text = Language.MainContextMenu.ChangeIcon;
            this.changeIconToolStripMenuItem.Click += new System.EventHandler(this.changeIconToolStripMenuItem_Click);
            // 
            // dockFolderToolStripMenuItem
            // 
            this.dockFolderToolStripMenuItem.Name = "dockFolderToolStripMenuItem";
            this.dockFolderToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.dockFolderToolStripMenuItem.Text = Language.MainContextMenu.DockFolder;
            this.dockFolderToolStripMenuItem.Click += new System.EventHandler(this.dockFolderToolStripMenuItem_Click);
            //
            // stackFolderToolStripMenuItem
            //
            this.stackFolderToolStripMenuItem.Name = "stackFolderToolStripMenuItem";
            this.stackFolderToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.stackFolderToolStripMenuItem.Text = "Folder Stack";
            this.stackFolderToolStripMenuItem.Click += new System.EventHandler(this.stackFolderToolStripMenuItem_Click);
            // 
            // blankIconToolStripMenuItem
            // 
            this.blankIconToolStripMenuItem.Name = "blankIconToolStripMenuItem";
            this.blankIconToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.blankIconToolStripMenuItem.Text = Language.MainContextMenu.blankIcon;
            this.blankIconToolStripMenuItem.Click += new System.EventHandler(this.blankIconToolStripMenuItem_Click);
            // 
            // TransparentObject
            // 
            this.ClientSize = new System.Drawing.Size(1, 1);
            this.dwellTimer = new System.Windows.Forms.Timer(this.components);
            this.dwellTimer.Tick += new EventHandler(dwellTimer_Tick);
            this.dwellTimer.Interval = 10;

            this.MouseEnter += new EventHandler(BackgroundObject_MouseEnter);
            this.MouseLeave += new EventHandler(BackgroundObject_MouseLeave);
            this.MouseDown += new MouseEventHandler(BackgroundObject_MouseDown);
            this.MouseUp += new MouseEventHandler(BackgroundObject_MouseUp);
            this.MouseMove += new MouseEventHandler(BackgroundObject_MouseMove);
            this.MouseWheel += new MouseEventHandler(BackgroundObject_MouseWheel);
            this.MouseHover += new EventHandler(BackgroundObject_MouseHover);

            this.DragEnter += new DragEventHandler(BackgroundObject_DragEnter);
            this.DragDrop += new DragEventHandler(BackgroundObject_DragDrop);

            this.KeyDown += new KeyEventHandler(BackgroundObject_KeyDown);

            this.Deactivate += new EventHandler(BackgroundObject_Deactivate);

            this.ContextMenuStrip = contextMenuStrip;

            this.Name = "TransparentObject";
            this.contextMenuStrip.ResumeLayout(false);
            this.ResumeLayout(false);
        }

        void BackgroundObject_Deactivate(object sender, EventArgs e)
        {
            ParentObject.itemDeactivated();
        }

        protected override void WndProc(ref Message m)
        {
            //if (m.Msg == Constants.WindowMessageConstants.WM_PARENTNOTIFY)
            //{
            //    Console.WriteLine("Parent Notified");
            //}
            base.WndProc(ref m);
        }

        #endregion


        #region Variables

        public String SectionName;

        Point previousMousePosition;

        /// <summary> 
        /// Use to store the difference between the global positon of the mouse cursor and the global position of an object.
        /// </summary>
        private Point StoredMouseOffset;
        public Boolean DockCanHide;

        #endregion

        #region Constructor

        /// <summary> 
        /// Create an alpha blended form with a transparent background that is movable and responds to mouse and key inputs.
        /// </summary>
        public BackgroundObject(MainForm TheParent, LanguageLoader.LanguageLoader LanguageData, SettingsLoader.SettingsLoader SettingsData, String Section, Size InitialSize)
        {
            Language = LanguageData;
            DockSettings = SettingsData;
            ParentObject = TheParent;
            SectionName = Section;
            ObjectSize = InitialSize;

            InitializeComponent();

            DoubleBuffered = true;
            this.AnimationTimer.Tick += new System.EventHandler(this.AnimationTimer_Tick);
            AnimationTimer.Interval = DockSettings.General.AnimationInterval;

            ShowInTaskbar = false;
            AllowDrop = true;

            LeftMouseButtonDown = false;
            ThisObjectMovedWithLeftMouse = false;

            ObjectBitmap = new Bitmap(1, 1, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
            OverlayBitmap = new Bitmap(1, 1, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
            ObjectOpacity = DockSettings.Background.DefaultOpacity;

            SetZLevel();

            SetBitmap();
        }

        public void SetZLevel()
        {
            if (DockSettings.General.zLevel == "Always On Bottom")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else if (DockSettings.General.zLevel == "Normal")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_NOTOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_TOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
        }

        private int TargetOpacity;
        private double OpacityRateOfChange;
        private double tempNewObjectOpacity;

        public void AnimateOpacity(int NewOpacity, int Duration)
        {
            if (Duration <= 0)
                return;

            TargetOpacity = NewOpacity;
            OpacityRateOfChange = ((double)NewOpacity - (double)ObjectOpacity) / ((double)Duration / (double)AnimationTimer.Interval);

            tempNewObjectOpacity = ObjectOpacity;
            AnimationTimer.Start();
        }

        private void AnimationTimer_Tick(object sender, EventArgs e)
        {
            tempNewObjectOpacity = tempNewObjectOpacity + OpacityRateOfChange;
            if ((OpacityRateOfChange > 0 && tempNewObjectOpacity > TargetOpacity) ||
                (OpacityRateOfChange < 0 && tempNewObjectOpacity < TargetOpacity))
                tempNewObjectOpacity = TargetOpacity;

            //DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, this.Location.X, this.Location.Y, false, 0, 0, 0, 0, true, (int)tempNewObjectOpacity);
            UpdateOpacity((int)tempNewObjectOpacity);

            if ((OpacityRateOfChange < 0 && ObjectOpacity <= TargetOpacity) ||
                (OpacityRateOfChange > 0 && ObjectOpacity >= TargetOpacity))
            {
                AnimationTimer.Stop();
                AnimationTimer.Enabled = false;
            }
        }

        public void UpdateAnimationTimerInterval(int NewInterval)
        {
            AnimationTimer.Interval = NewInterval;
        }

        public void SetBitmap()
        {
            Bitmap tempBitmap;
            try
            {
                if (DockSettings.Background.Path.StartsWith(@".\"))
                {
                    tempBitmap = new Bitmap(Application.StartupPath + @DockSettings.Background.Path.Substring(1, DockSettings.Background.Path.Length - 1));
                }
                else
                {
                    tempBitmap = new Bitmap(@DockSettings.Background.Path);
                }
            }
            catch (Exception)
            {
                tempBitmap = new Bitmap(ImageResources.sapphire_ring);
                //MessageBox.Show("Error creating bitmap from file. CentreObject.");
            }

            SetBitmapManaged(ref tempBitmap);
        }

        #endregion

        #region Object Events

        /// <summary> 
        /// Move the text label when this object is moved.
        /// </summary>
        protected override void OnMove(EventArgs e)
        {
            base.OnMove(e);
        }

        void BackgroundObject_MouseHover(object sender, EventArgs e)
        {
            this.Activate();
        }

        void BackgroundObject_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left)
                ParentObject.RotateDock(-1.0);
            else if (e.KeyCode == Keys.Right)
                ParentObject.RotateDock(1.0);
            else if (e.KeyCode == Keys.Down)
                ParentObject.RotateDock(-1.0);
            else if (e.KeyCode == Keys.Up)
                ParentObject.RotateDock(1.0);
        }

        /// <summary> 
        /// Change the cursor when using Drag and Drop
        /// </summary>
        void BackgroundObject_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Link;
        }

        /// <summary> 
        /// Add files to the dock using Drag and Drop
        /// </summary>
        void BackgroundObject_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = e.Data.GetData(DataFormats.FileDrop) as string[];

            if (DockSettings.General.IconReplacementMode && files != null && files.Length == 1 && files[0].ToUpper().EndsWith(@".PNG"))
            {
                ChangeIconDialog.FileName = files[0];
                ChangeIconDialog_FileOk(this, null);
            }
            else if (files != null)
            {
                ParentObject.CentreObject.Activate();
                for (int i = 0; i < files.Length; i++)
                {

                    ParentObject.AddDockItem("[link]", files[i], "Link", "", "");
                }
                ParentObject.PositionCurrentLevel();
            }
        }

        void BackgroundObject_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta < 0)
                ParentObject.RotateDock(1.0);
            else
                ParentObject.RotateDock(-1.0);
        }

        void BackgroundObject_MouseMove(object sender, MouseEventArgs e)
        {
            if (DockSettings.General.lockDockPosition == false && LeftMouseButtonDown)
            {
                if (ThisObjectMovedWithLeftMouse == true || Math.Abs(Cursor.Position.X - previousMousePosition.X) > DockSettings.General.ClickSensitivity
                    || Math.Abs(Cursor.Position.Y - previousMousePosition.Y) > DockSettings.General.ClickSensitivity)
                {
                    ThisObjectMovedWithLeftMouse = true;

                    this.Location = new Point(Cursor.Position.X - StoredMouseOffset.X, Cursor.Position.Y - StoredMouseOffset.Y);

                    if (ParentObject != null)
                    {
                        ParentObject.ChildMoved(this);
                    }
                }
            }
        }

        void BackgroundObject_MouseUp(object sender, MouseEventArgs e)
        {
            if (ThisObjectMovedWithLeftMouse == false)
            {
                if (e.Button == MouseButtons.Left && LeftMouseButtonDown)
                {
                    LeftMouseButtonDown = false;
                }
            }
            else
            {
                ThisObjectMovedWithLeftMouse = false;

                if (e.Button == MouseButtons.Left && LeftMouseButtonDown)
                {
                    LeftMouseButtonDown = false;
                }
            }
        }

        void BackgroundObject_MouseDown(object sender, MouseEventArgs e)
        {
            this.Activate();
            this.BringToFront();
            if (e.Button == MouseButtons.Left)
            {
                LeftMouseButtonDown = true;
                ThisObjectMovedWithLeftMouse = false;
                StoredMouseOffset = new Point(Cursor.Position.X - this.Location.X, Cursor.Position.Y - this.Location.Y);
                previousMousePosition = Cursor.Position;
            }
        }

        void BackgroundObject_MouseEnter(object sender, EventArgs e)
        {
            DockCanHide = false;
            //dwellTimer.Stop();
            this.BringToFront();
        }

        private int dwellTime = 0;

        void BackgroundObject_MouseLeave(object sender, EventArgs e)
        {
            if (DockSettings.Toggling.AutoHide)
            {
                DockCanHide = true;
                dwellTime = DockSettings.Toggling.dwellTime * -1;
                dwellTimer.Start();
            }
        }

        private void dwellTimer_Tick(object sender, EventArgs e)
        {
            if (DockCanHide == false)
            {
                dwellTimer.Stop();
                dwellTime = 0;
                return;
            }

            dwellTime += dwellTimer.Interval;

            if (dwellTime >= DockSettings.Toggling.dwellTime)
            {
                dwellTime = 0;
                dwellTimer.Stop();
                ParentObject.ToggleVisiblity();
            }
        }

        #endregion

        #region Context Menu

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.ChildRequestAction("QUIT");
        }
        //
        // Add a Folder to the Dock
        //
        private void dockFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.CentreObject.Activate();
            ParentObject.AddDockItem("[dockfolder]", "", Language.MainContextMenu.DockFolder, DockSettings.Folders.DockFolderImagePath, Language.MainContextMenu.DockFolder);
            ParentObject.PositionCurrentLevel();
        }
        //
        // Add a Blank Item to the Dock
        //
        private void blankIconToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.CentreObject.Activate();
            ParentObject.AddDockItem("[blanklink]", "", "Link", "", "");
            ParentObject.PositionCurrentLevel();
        }
        //
        // Add a Folder Stack to the Dock
        //
        private void stackFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.CentreObject.Activate();
            ParentObject.AddDockItem("[stack]", ".\\System\\Miscellaneous\\StandaloneStack.Exe", "Stack", "", "Change Me Now!");
            ParentObject.PositionCurrentLevel();
        }

        private void hideToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.HideAll();
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.ShowSettingsPanel();
        }

        private void pauseMouseHotkeysToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.pauseMouseHotkeysToolStripMenuItem.Checked = pauseMouseHotkeysToolStripMenuItem.Checked;
            ParentObject.SetMouseKeys();
        }

        private void changeIconToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeIconDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            ChangeIconDialog.ShowDialog();
        }

        private void iconReplacementModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DockSettings.General.IconReplacementMode = iconReplacementModeToolStripMenuItem.Checked;
            DockSettings.SetEntry("General", "IconReplacementMode", iconReplacementModeToolStripMenuItem.Checked.ToString());

            if (DockSettings.General.IconReplacementMode)
            {
                FileOps.FileOps FileOperations = new FileOps.FileOps(IntPtr.Zero, Language, DockSettings);
                FileOperations.Open(Application.StartupPath + @"\System\Icons", @"", System.Diagnostics.ProcessWindowStyle.Normal, this.Handle);
            }
        }

        private void toggleAutoHide_Click(object sender, EventArgs e)
        {
            DockSettings.Toggling.AutoHide = toggleAutoHide.Checked;
        }

        private void contextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            DockCanHide = false; // this.ParentObject.BackgroundObject.dwellTimer.Stop();
            iconReplacementModeToolStripMenuItem.Checked = DockSettings.General.IconReplacementMode;
            pauseMouseHotkeysToolStripMenuItem.Checked = ParentObject.pauseMouseHotkeysToolStripMenuItem.Checked;
            toggleAutoHide.Checked = DockSettings.Toggling.AutoHide;
        }

        private void contextMenuStrip_MouseEnter(object sender, EventArgs e)
        {
            DockCanHide = false;
        }

        private void ChangeIconDialog_FileOk(object sender, CancelEventArgs e)
        {
            String NewImagePath;
            if (ChangeIconDialog.FileName.StartsWith(Application.StartupPath + @"\"))
            {
                NewImagePath = @"." + ChangeIconDialog.FileName.Substring(Application.StartupPath.Length, ChangeIconDialog.FileName.Length - Application.StartupPath.Length);
            }
            else
            {
                NewImagePath = ChangeIconDialog.FileName;
            }

            DockSettings.SetEntry("Background", "Path", NewImagePath);
            DockSettings.Background.Path = NewImagePath;

            SetBitmap();
            Size ObjectSize = this.ObjectSize;
            DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, false, 0);
        }

        #endregion
    }

    public partial class LabelObject : PerPixelAlphaForms.LabelPerPixelAlphaForm
    {
        #region Windows Form Designer

        private System.ComponentModel.IContainer components;

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.SuspendLayout();
            this.Deactivate += new EventHandler(LabelObject_Deactivate);
            // 
            // TransparentObject
            // 
            this.ClientSize = new System.Drawing.Size(1, 1);
            this.Name = "TransparentObject";
            this.ResumeLayout(false);
        }

        void LabelObject_Deactivate(object sender, EventArgs e)
        {
            ParentObject.itemDeactivated();
        }

        #endregion

        #region Variables

        String LabelText;
        Size LabelSize;
        Point CentrePoint;
        MainForm ParentObject;

        #endregion

        #region Constructor

        /// <summary> 
        /// Create an alpha blended form with a transparent background that is movable and responds to mouse and key inputs.
        /// </summary>
        public LabelObject(MainForm theParent, SettingsLoader.SettingsLoader SettingsData, Size DisplaySize)
        {
            ParentObject = theParent;
            DockSettings = SettingsData;
            LabelSize = DisplaySize;

            InitializeComponent();

            DoubleBuffered = true;
            this.AnimationTimer.Tick += new System.EventHandler(this.AnimationTimer_Tick);
            AnimationTimer.Interval = DockSettings.General.AnimationInterval;

            SetZLevel();
            ShowInTaskbar = false;
            AllowDrop = false;

            ObjectOpacity = DockSettings.Labels.DefaultOpacity;
        }

        public void SetZLevel()
        {
            if (DockSettings.General.zLevel == "Always On Bottom")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else if (DockSettings.General.zLevel == "Normal")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_NOTOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_TOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
        }

        private int TargetOpacity;
        private double OpacityRateOfChange;
        private double tempNewObjectOpacity;

        public void AnimateOpacity(int NewOpacity, int Duration)
        {
            if (Duration <= 0)
                return;

            TargetOpacity = NewOpacity;
            OpacityRateOfChange = ((double)NewOpacity - (double)ObjectOpacity) / ((double)Duration / (double)AnimationTimer.Interval);

            tempNewObjectOpacity = ObjectOpacity;
            AnimationTimer.Start();
        }

        private void AnimationTimer_Tick(object sender, EventArgs e)
        {
            tempNewObjectOpacity = tempNewObjectOpacity + OpacityRateOfChange;
            if ((OpacityRateOfChange > 0 && tempNewObjectOpacity > TargetOpacity) ||
                (OpacityRateOfChange < 0 && tempNewObjectOpacity < TargetOpacity))
                tempNewObjectOpacity = TargetOpacity; 

            UpdateOpacity((int)tempNewObjectOpacity);

            if ((OpacityRateOfChange < 0 && ObjectOpacity <= TargetOpacity) ||
                (OpacityRateOfChange > 0 && ObjectOpacity >= TargetOpacity))
            {
                AnimationTimer.Stop();
                AnimationTimer.Enabled = false;
            }
        }

        public void UpdateAnimationTimerInterval(int NewInterval)
        {
            AnimationTimer.Interval = NewInterval;
        }

        public void RedrawBitmap()
        {
            SetBitmap(LabelText, CentrePoint);
        }

        public void SetBitmap(String DisplayText, Point DisplayAtCentrePoint)
        {
            if (DisplayAtCentrePoint == null)
                return;
            else
                CentrePoint = DisplayAtCentrePoint;

            try
            {
                LabelText = DisplayText;

                Bitmap tempBitmap;

                if (DisplayText != null && DisplayText.Length > 0)
                {
                    Font drawFont = DockSettings.Labels.FontName;
                    Color drawBrush = DockSettings.Labels.FontColor;
                    Color drawBrushShadow = DockSettings.Labels.ShadowColor;

                    tempBitmap = (Bitmap)ImageOperations.FancyText.ImageFromText(DisplayText, drawFont, drawBrush, drawBrushShadow, 
                        DockSettings.Labels.ShadowSize, DockSettings.Labels.ShadowDarkness);
                }
                else
                {
                    tempBitmap = new Bitmap(1, 1);
                }

                SetBitmapManaged(ref tempBitmap);

                DrawBitmapManaged(tempBitmap.Width, tempBitmap.Height, true, DisplayAtCentrePoint.X - tempBitmap.Width / 2,
                    DisplayAtCentrePoint.Y, false, 0, 0, 0, 0, false, 0);

                tempBitmap.Dispose();
            }
            catch (Exception)
            {
                //MessageBox.Show("Error creating bitmap from file. LabelObject: " + ee.ToString());
            }
        }

        #endregion
    }

    public partial class DockItemObject : PerPixelAlphaForms.DockItemPerPixelAlphaForm
    {
        #region Windows Form Designer

        private ContextMenuStrip contextMenuStrip;
        private System.ComponentModel.IContainer components;
        private ToolStripMenuItem removeToolStripMenuItem;
        private ToolStripMenuItem changeIconToolStripMenuItem;
        private ToolStripMenuItem itemSettingsToolStripMenuItem;
        private OpenFileDialog ChangeIconDialog;

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.contextMenuStrip = new System.Windows.Forms.ContextMenuStrip(this.components);
            this.removeToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.changeIconToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.itemSettingsToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.ChangeIconDialog = new System.Windows.Forms.OpenFileDialog();
            this.contextMenuStrip.SuspendLayout();
            this.SuspendLayout();
            // 
            // contextMenuStrip
            // 
            this.contextMenuStrip.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.changeIconToolStripMenuItem, this.removeToolStripMenuItem, this.itemSettingsToolStripMenuItem});
            this.contextMenuStrip.Name = "contextMenuStrip";
            this.contextMenuStrip.Size = new System.Drawing.Size(153, 48);
            // 
            // changeIconStripMenuItem
            // 
            this.changeIconToolStripMenuItem.Name = "changeIconToolStripMenuItem";
            this.changeIconToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.changeIconToolStripMenuItem.Text = Language.MainContextMenu.ChangeIcon;
            this.changeIconToolStripMenuItem.Click += new System.EventHandler(this.changeIconToolStripMenuItem_Click);
            // 
            // removeToolStripMenuItem
            // 
            this.removeToolStripMenuItem.Name = "removeToolStripMenuItem";
            this.removeToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.removeToolStripMenuItem.Text = Language.MainContextMenu.RemoveWord;
            this.removeToolStripMenuItem.Click += new System.EventHandler(this.removeToolStripMenuItem_Click);
            // 
            // itemSettingsToolStripMenuItem
            // 
            this.itemSettingsToolStripMenuItem.Name = "itemSettingsToolStripMenuItem";
            this.itemSettingsToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.itemSettingsToolStripMenuItem.Text = Language.MainContextMenu.ItemSettings;
            this.itemSettingsToolStripMenuItem.Click += new System.EventHandler(this.itemSettingsToolStripMenuItem_Click);
            // 
            // OpenDataPointsDialog
            // 
            this.ChangeIconDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            this.ChangeIconDialog.Filter = "PNG|*.png";
            this.ChangeIconDialog.Multiselect = false;
            this.ChangeIconDialog.Title = Language.MainContextMenu.ChangeIcon;
            this.ChangeIconDialog.FileOk += new System.ComponentModel.CancelEventHandler(this.ChangeIconDialog_FileOk);
            // 
            // TransparentObject
            // 
            this.ClientSize = new System.Drawing.Size(1, 1);

            this.MouseEnter += new EventHandler(DockItem_MouseEnter);
            this.MouseDown += new MouseEventHandler(DockItem_MouseDown);
            this.MouseUp += new MouseEventHandler(DockItem_MouseUp);
            this.MouseMove += new MouseEventHandler(DockItem_MouseMove);
            this.MouseWheel += new MouseEventHandler(DockItem_MouseWheel);
            this.MouseHover += new EventHandler(DockItem_MouseHover);
            this.MouseLeave += new EventHandler(DockItemObject_MouseLeave);

            this.DragEnter += new DragEventHandler(DockItem_DragEnter);
            this.DragDrop += new DragEventHandler(DockItem_DragDrop);

            this.KeyDown += new KeyEventHandler(DockItem_KeyDown);
            //this.ContextMenuStrip = contextMenuStrip;
            this.Shown += new EventHandler(DockItemObject_Shown);

            this.Deactivate += new EventHandler(DockItemObject_Deactivate);

            this.Name = "TransparentObject";
            this.contextMenuStrip.ResumeLayout(false);
            this.ResumeLayout(false);
        }

        void DockItemObject_Shown(object sender, EventArgs e)
        {
            //this.Owner = ParentObject.CentreObject;
            //this.Activate();
            //this.BringToFront();
            //this.MyLabel.Activate();
            //this.MyLabel.BringToFront();
        }

        void DockItemObject_Deactivate(object sender, EventArgs e)
        {
            if (this.Disposing == false)
                ParentObject.itemDeactivated();
        }

        #endregion

        #region Variables

        DockItemSettingsLoader.DockItemSettingsLoader DockItemSettings;
        public String DockItemSectionName;
        public int PreviousOpacity;
        public FileOps.FileOps FileOperations;
        public int dockOrderNum;
        Point previousMousePosition;

        /// <summary> 
        /// Use to store the difference between the global positon of the mouse cursor and the global position of an object.
        /// </summary>
        private Point StoredMouseOffset;

        public LabelObject MyLabel;

        #endregion

        #region Constructor

        /// <summary> 
        /// Create an alpha blended form with a transparent background that is movable and responds to mouse and key inputs.
        /// </summary>
        public DockItemObject(MainForm TheParent, LanguageLoader.LanguageLoader LanguageData, SettingsLoader.SettingsLoader SettingsData,
            DockItemSettingsLoader.DockItemSettingsLoader DockItemData, String SectionName, Size InitialSize, String Path)
        {
            Language = LanguageData;
            DockSettings = SettingsData;
            ParentObject = TheParent;
            DockItemSettings = DockItemData;
            DockItemSectionName = SectionName;
            ObjectSize = InitialSize;
            StoredMouseOffset = new Point(0, 0);
            dockOrderNum = 0;
            RetrieveDockItemOrder();

            InitializeFileOps();

            InitializeComponent();

            this.AnimationTimer.Tick += new System.EventHandler(this.AnimationTimer_Tick);
            AnimationTimer.Interval = DockSettings.General.AnimationInterval;

            //TopMost = true;
            SetZLevel();

            LeftMouseButtonDown = false;
            ThisObjectMovedWithLeftMouse = false;

            ObjectOpacity = DockSettings.DockItems.DefaultOpacity;
            PreviousOpacity = ObjectOpacity;

            SetBitmap();

            MyLabel = new LabelObject(ParentObject, SettingsData, this.ObjectSize);
            MyLabel.Owner = this;
            //MyLabel.Location = new Point(this.Location.X, this.Location.Y + ObjectSize.Height);
            //MyLabel.BringToFront();
        }

        public void SetZLevel()
        {
            if (DockSettings.General.zLevel == "Always On Bottom")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else if (DockSettings.General.zLevel == "Normal")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_NOTOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_TOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
        }

        public void InitializeFileOps()
        {
            if (DockItemSectionName != null && DockItemSectionName != "")
            {
                if (DockItemSettings.GetEntry(DockItemSectionName, "Action") == "[link]")
                {
                    FileOperations = new FileOps.FileOps(IntPtr.Zero, @DockItemSettings.GetEntry(DockItemSectionName, "Args"), Language, DockSettings);
                    String tempName = DockItemSettings.GetEntry(DockItemSectionName, "Name");
                    if (tempName.Length > 0)
                    {
                        FileOperations.Name = tempName;
                    }
                }
                else if (DockItemSettings.GetEntry(DockItemSectionName, "Action") == "[dockfolder]")
                {
                    FileOperations = new FileOps.FileOps(IntPtr.Zero, Language, DockSettings);
                    String tempName = DockItemSettings.GetEntry(DockItemSectionName, "Name");
                    if (tempName.Length > 0)
                    {
                        FileOperations.Name = tempName;
                    }
                }
            }
        }

        private int TargetOpacity;
        private double OpacityRateOfChange;
        private double tempNewObjectOpacity;

        public void AnimateOpacity(int NewOpacity, int Duration)
        {
            if (Duration <= 0)
                return;

            TargetOpacity = NewOpacity;
            OpacityRateOfChange = ((double)NewOpacity - (double)ObjectOpacity) / ((double)Duration / (double)AnimationTimer.Interval);

            tempNewObjectOpacity = ObjectOpacity;
            AnimationTimer.Start();
        }

        private void AnimationTimer_Tick(object sender, EventArgs e)
        {
            tempNewObjectOpacity = tempNewObjectOpacity + OpacityRateOfChange;
            if ((OpacityRateOfChange > 0 && tempNewObjectOpacity > TargetOpacity) ||
                (OpacityRateOfChange < 0 && tempNewObjectOpacity < TargetOpacity))
                tempNewObjectOpacity = TargetOpacity;

            UpdateOpacity((int)tempNewObjectOpacity);

            if (ObjectOpacity == 0 || ObjectOpacity == 255 || (OpacityRateOfChange < 0 && ObjectOpacity <= TargetOpacity) ||
                (OpacityRateOfChange > 0 && ObjectOpacity >= TargetOpacity))
            {
                AnimationTimer.Stop();
                AnimationTimer.Enabled = false;
            }
        }

        public void UpdateAnimationTimerInterval(int NewInterval)
        {
            AnimationTimer.Interval = NewInterval;
            MyLabel.UpdateAnimationTimerInterval(NewInterval);
        }

        public void RetrieveDockItemOrder()
        {
            if (DockItemSectionName != null && DockItemSectionName != "")
            {
                try
                {
                    dockOrderNum = int.Parse(DockItemSettings.GetEntry(DockItemSectionName, "Order"));
                }
                catch (Exception)
                {
                    dockOrderNum = 0;
                }
            }
            else
            {
                dockOrderNum = 0;
            }
        }

        public void SetDockItemOrder(int NewOrderNum)
        {
            dockOrderNum = NewOrderNum;
            DockItemSettings.SetEntry(DockItemSectionName, "Order", NewOrderNum.ToString());
        }

        #endregion

        public void SetBitmap()
        {
            Bitmap tempBitmap;
            try
            {
                if (DockItemSectionName != null && DockItemSectionName.Length > 0)
                {
                    String ImagePath = @DockItemSettings.GetEntry(DockItemSectionName, "ImagePath");

                    if (ImagePath.Length > 0)
                    {
                        if (ImagePath.StartsWith(@".\"))
                        {
                            tempBitmap = new Bitmap(Application.StartupPath + @ImagePath.Substring(1, ImagePath.Length - 1));
                        }
                        else
                        {
                            tempBitmap = new Bitmap(@ImagePath);
                        }
                    }
                    else
                    {
                        tempBitmap = new Bitmap(FileOperations.RepresentativeImage);
                    }
                }
                else
                {
                    tempBitmap = new Bitmap(FileOperations.RepresentativeImage);
                }
            }
            catch (Exception)
            {
                tempBitmap = new Bitmap(ImageResources.MissingIcon);
            }

            SetBitmapManaged(ref tempBitmap);
        }

        #region Object Events

        public void SetLabel(String DisplayText)
        {
            Point DisplayAtCentrePoint = new Point(this.Location.X + ObjectSize.Width / 2, this.Location.Y + ObjectSize.Height);
            MyLabel.SetBitmap(DisplayText, DisplayAtCentrePoint);
        }

        /// <summary> 
        /// Move the text label when this object is moved.
        /// </summary>
        protected override void OnMove(EventArgs e)
        {
            //if (MyLabel != null)
            //{
            //    Size tempMyLabelSize = MyLabel.ObjectSize;
            //    MyLabel.Location = new Point(this.Location.X + ObjectSize.Width / 2 - tempMyLabelSize.Width / 2, this.Location.Y + ObjectSize.Height);
            //}
            base.OnMove(e);
        }

        void DockItem_MouseHover(object sender, EventArgs e)
        {
            this.Activate();
        }

        void DockItem_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left)
                ParentObject.RotateDock(-1.0);
            else if (e.KeyCode == Keys.Right)
                ParentObject.RotateDock(1.0);
            else if (e.KeyCode == Keys.Down)
                ParentObject.RotateDock(-1.0);
            else if (e.KeyCode == Keys.Up)
                ParentObject.RotateDock(1.0);
            else if (e.KeyCode == Keys.Delete)
                ParentObject.RemoveDockItem(this);
        }

        /// <summary> 
        /// Add files to the dock using Drag and Drop
        /// </summary>
        void DockItem_DragDrop(object sender, DragEventArgs e)
        {
            //string[] files2 = e.Data.GetData(DataFormats.SymbolicLink) as string[];

            //Console.WriteLine(files2[0]);


            string[] files = e.Data.GetData(DataFormats.FileDrop) as string[];
            if (DockSettings.General.IconReplacementMode && files != null && files.Length == 1 && files[0].ToUpper().EndsWith(@".PNG"))
            {
                ChangeIconDialog.FileName = files[0];
                ChangeIconDialog_FileOk(this, null);
            }
            else if (files != null)
            {
                ParentObject.CentreObject.Activate();
                for (int i = 0; i < files.Length; i++)
                {
                    ParentObject.AddDockItem("[link]", files[i], "Link", "", "");
                    //ParentObject.CentreObject.Activate();
                }
                ParentObject.PositionCurrentLevel();
            }
        }

        /// <summary> 
        /// Change the cursor when using Drag and Drop
        /// </summary>
        void DockItem_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Link;
            //base.OnDragEnter(e);
        }

        void DockItem_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta < 0)
                ParentObject.RotateDock(1.0);
            else
                ParentObject.RotateDock(-1.0);
        }

        void DockItem_MouseMove(object sender, MouseEventArgs e)
        {
            if (LeftMouseButtonDown)
            {
                if (ThisObjectMovedWithLeftMouse == true || Math.Abs(Cursor.Position.X - previousMousePosition.X) > DockSettings.General.ClickSensitivity
                    || Math.Abs(Cursor.Position.Y - previousMousePosition.Y) > DockSettings.General.ClickSensitivity)
                {
                    ThisObjectMovedWithLeftMouse = true;

                    this.Location = new Point(Cursor.Position.X - StoredMouseOffset.X, Cursor.Position.Y - StoredMouseOffset.Y);

                    if (ParentObject != null)
                        ParentObject.ChildMoving(this);
                }
            }
        }

        void DockItem_MouseUp(object sender, MouseEventArgs e)
        {
            if (ThisObjectMovedWithLeftMouse == false)
            {
                if (e.Button == MouseButtons.Left && LeftMouseButtonDown)
                {
                    LeftMouseButtonDown = false;

                    if (ParentObject.DockIsVisible || DockSettings.General.zLevel == @"Always On Bottom")
                    {
                        //DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, PreviousOpacity);
                        DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, DockSettings.DockItems.DefaultOpacity);
                        PreviousOpacity = ObjectOpacity;
                    }
                    else
                    {
                        return;
                    }

                    if (DockItemSectionName != null && DockItemSettings.GetEntry(DockItemSectionName, @"Action") == @"[link]")
                    {
                        if (DockSettings.DockItems.HideDockAfterExecution)
                            ParentObject.HideAll();

                        System.Diagnostics.ProcessWindowStyle runAs = System.Diagnostics.ProcessWindowStyle.Normal;
                        String runAsArg = DockItemSettings.GetEntry(DockItemSectionName, @"RunAs");

                        if (runAsArg == null || (runAsArg != @"Normal" && runAsArg != "Minimized" && runAsArg != "Maximized"))
                        {
                            DockItemSettings.SetEntry(DockItemSectionName, @"RunAs", @"Normal");
                            runAs = System.Diagnostics.ProcessWindowStyle.Normal;
                        }
                        else
                        {
                            if (runAsArg == @"Maximized")
                                runAs = System.Diagnostics.ProcessWindowStyle.Maximized;
                            else if (runAsArg == @"Minimized")
                                runAs = System.Diagnostics.ProcessWindowStyle.Minimized;
                            else
                                runAs = System.Diagnostics.ProcessWindowStyle.Normal;
                        }

                        FileOperations.Open(@FileOperations._Path, DockItemSettings.GetEntry(DockItemSectionName, @"ArgsParams"), runAs, this.Handle);
                    }
                    else if (DockItemSectionName != null && DockItemSettings.GetEntry(DockItemSectionName, @"Action") == @"[dockfolder]")
                    {
                        ParentObject.ShowLevel(DockItemSectionName + @"-");
                    }
                }
                else if (e.Button == MouseButtons.Right)
                {
                    ParentObject.BackgroundObject.DockCanHide = false; // this.ParentObject.BackgroundObject.dwellTimer.Stop();
                    if (DockSettings.DockItems.ShowExplorerContextMenus && DockItemSectionName != null && DockItemSettings.GetEntry(DockItemSectionName, @"Action") == @"[link]")
                    {
                        try
                        {
                            ShellExtension.ShellContextMenu ctxMnu = new ShellExtension.ShellContextMenu();
                            FileInfo[] arrFI = new FileInfo[1];
                            arrFI[0] = new FileInfo(DockItemSettings.GetEntry(DockItemSectionName, @"Args"));

                            String[] AddedItems = new String[3];
                            AddedItems[0] = Language.MainContextMenu.ChangeIcon;
                            AddedItems[1] = Language.MainContextMenu.RemoveWord;
                            AddedItems[2] = Language.MainContextMenu.ItemSettings;

                            int SelectedAddedItem = ctxMnu.ShowContextMenu(arrFI, Cursor.Position, AddedItems);

                            if (SelectedAddedItem >= 0)
                            {
                                switch (SelectedAddedItem)
                                {
                                    case 0:
                                        changeIconToolStripMenuItem_Click(this, null);
                                        break;
                                    case 1:
                                        removeToolStripMenuItem_Click(this, null);
                                        break;
                                    case 2:
                                        itemSettingsToolStripMenuItem_Click(this, null);
                                        break;
                                    default:
                                        break;
                                }
                            }
                            else if (SelectedAddedItem == -2)
                            {
                                //Console.WriteLine("here: " + SelectedAddedItem);
                                this.ParentObject.BackgroundObject.DockCanHide = false;
                                contextMenuStrip.Show();
                                contextMenuStrip.Left = MousePosition.X;
                                contextMenuStrip.Top = MousePosition.Y;
                            }

                        }
                        catch (Exception)
                        {
                            //Console.WriteLine("Shell Extension Error");
                            this.ParentObject.BackgroundObject.DockCanHide = false;
                            contextMenuStrip.Show();
                            contextMenuStrip.Left = MousePosition.X;
                            contextMenuStrip.Top = MousePosition.Y;
                        }
                    }
                    else
                    {
                        this.ParentObject.BackgroundObject.DockCanHide = false;
                        contextMenuStrip.Show();
                        contextMenuStrip.Left = MousePosition.X;
                        contextMenuStrip.Top = MousePosition.Y;
                    }
                }
            }
            else
            {
                ThisObjectMovedWithLeftMouse = false;

                if (e.Button == MouseButtons.Left && LeftMouseButtonDown)
                {
                    LeftMouseButtonDown = false;

                    if (ParentObject.DockIsVisible || DockSettings.General.zLevel == @"Always On Bottom")
                    {
                        //DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, PreviousOpacity);
                        DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, DockSettings.DockItems.DefaultOpacity);
                        PreviousOpacity = ObjectOpacity;
                    }

                    ParentObject.ChildMoved(this);
                }
            }
        }

        void DockItem_MouseDown(object sender, MouseEventArgs e)
        {
            this.Activate();
            this.BringToFront();
            if (e.Button == MouseButtons.Left)
            {
                LeftMouseButtonDown = true;
                ThisObjectMovedWithLeftMouse = false;
                PreviousOpacity = ObjectOpacity;
                DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, ObjectOpacity / 2);
                StoredMouseOffset = new Point(Cursor.Position.X - this.Location.X, Cursor.Position.Y - this.Location.Y);
                previousMousePosition = Cursor.Position;
                //this.BringToFront();
            }
        }

        void DockItem_MouseEnter(object sender, EventArgs e)
        {
            this.BringToFront();
            this.ParentObject.BackgroundObject.DockCanHide = false; // this.ParentObject.BackgroundObject.dwellTimer.Stop();

            if ((DockSettings.Labels.ShowLabels) && (DockSettings.Labels.ShowLabelsOnMouseOverOnly))
            {
                try
                {
                    SetLabel(FileOperations.Name);
                }
                catch (Exception)
                {
                    SetLabel("");
                }
            }
        }

        void DockItemObject_MouseLeave(object sender, EventArgs e)
        {
            if ((DockSettings.Labels.ShowLabels) && (DockSettings.Labels.ShowLabelsOnMouseOverOnly))
            {
                SetLabel("");
            }
        }

        #endregion

        #region Context Menu

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.ChildRequestAction("QUIT");
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.RemoveDockItem(this);
        }

        private void changeIconToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeIconDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            ChangeIconDialog.ShowDialog();
        }

        public void itemSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DockItemProperties itemSettings = new DockItemProperties(this, Language, DockSettings, DockItemSettings, DockItemSectionName);
            itemSettings.ShowDialog();
        }

        private void ChangeIconDialog_FileOk(object sender, CancelEventArgs e)
        {
            String NewImagePath;
            if (ChangeIconDialog.FileName.StartsWith(Application.StartupPath + @"\"))
            {
                NewImagePath = @"." + ChangeIconDialog.FileName.Substring(Application.StartupPath.Length, ChangeIconDialog.FileName.Length - Application.StartupPath.Length);
            }
            else
            {
                NewImagePath = ChangeIconDialog.FileName;
            }

            if (DockItemSectionName != null && DockItemSectionName != "")
            {
                DockItemSettings.SetEntry(DockItemSectionName, "ImagePath", NewImagePath);
            }

            SetBitmap();
            Size ObjectSize = this.ObjectSize;
            DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, false, 0);
        }


        public void SetBitmapRedraw()
        {
            SetBitmap();
            Size ObjectSize = this.ObjectSize;
            DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, false, 0);
        }

        #endregion
    }

    public partial class CentreObject : PerPixelAlphaForms.DockItemPerPixelAlphaForm
    {
        #region Windows Form Designer

        private ContextMenuStrip contextMenuStrip;
        private System.ComponentModel.IContainer components;
        private ToolStripMenuItem quitToolStripMenuItem;
        private ToolStripMenuItem addToolStripMenuItem;
        private ToolStripMenuItem toggleAutoHide;
        private ToolStripMenuItem hideToolStripMenuItem;
        private ToolStripMenuItem settingsToolStripMenuItem;
        private ToolStripMenuItem dockFolderToolStripMenuItem;
        private ToolStripMenuItem stackFolderToolStripMenuItem;
        private ToolStripMenuItem changeIconToolStripMenuItem;
        private ToolStripMenuItem iconReplacementModeToolStripMenuItem;
        private ToolStripMenuItem pauseMouseHotkeysToolStripMenuItem;
        private ToolStripMenuItem blankIconToolStripMenuItem;
        private OpenFileDialog ChangeIconDialog;

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.contextMenuStrip = new System.Windows.Forms.ContextMenuStrip(this.components);
            this.quitToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.addToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.toggleAutoHide = new System.Windows.Forms.ToolStripMenuItem();
            this.hideToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.dockFolderToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.stackFolderToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.blankIconToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.settingsToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.changeIconToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.iconReplacementModeToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.pauseMouseHotkeysToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.ChangeIconDialog = new System.Windows.Forms.OpenFileDialog();
            this.contextMenuStrip.SuspendLayout();
            this.SuspendLayout();
            // 
            // contextMenuStrip
            // 
            this.contextMenuStrip.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.hideToolStripMenuItem, this.addToolStripMenuItem, this.toggleAutoHide, this.changeIconToolStripMenuItem, this.iconReplacementModeToolStripMenuItem, 
                this.pauseMouseHotkeysToolStripMenuItem, this.settingsToolStripMenuItem, this.quitToolStripMenuItem});
            this.contextMenuStrip.Name = "contextMenuStrip";
            this.contextMenuStrip.Size = new System.Drawing.Size(153, 48);
            this.contextMenuStrip.Opening += new System.ComponentModel.CancelEventHandler(this.contextMenuStrip_Opening);
            this.contextMenuStrip.MouseEnter += new EventHandler(contextMenuStrip_MouseEnter);
            // 
            // quitToolStripMenuItem
            // 
            this.quitToolStripMenuItem.Name = "quitToolStripMenuItem";
            this.quitToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.quitToolStripMenuItem.Text = Language.MainContextMenu.QuitWord;
            this.quitToolStripMenuItem.Click += new System.EventHandler(this.quitToolStripMenuItem_Click);
            // 
            // addToolStripMenuItem
            // 
            this.addToolStripMenuItem.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.dockFolderToolStripMenuItem, this.blankIconToolStripMenuItem, this.stackFolderToolStripMenuItem});
            this.addToolStripMenuItem.Name = "addToolStripMenuItem";
            this.addToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.addToolStripMenuItem.Text = Language.MainContextMenu.AddWord;
            //
            //
            // Toggle Auto-Hide
            //
            this.toggleAutoHide.Name = "toggleAutoHide1";
            this.toggleAutoHide.Size = new System.Drawing.Size(152, 22);
            this.toggleAutoHide.Text = "AutoHide";
            this.toggleAutoHide.CheckOnClick = true;
            this.toggleAutoHide.Checked = DockSettings.Toggling.AutoHide;
            this.toggleAutoHide.Click += new System.EventHandler(this.toggleAutoHide_Click);
            // 
            // hideToolStripMenuItem
            // 
            this.hideToolStripMenuItem.Name = "hideToolStripMenuItem";
            this.hideToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.hideToolStripMenuItem.Text = Language.MainContextMenu.HideWord;
            this.hideToolStripMenuItem.Click += new System.EventHandler(this.hideToolStripMenuItem_Click);
            // 
            // settingsToolStripMenuItem
            // 
            this.settingsToolStripMenuItem.Name = "settingsToolStripMenuItem";
            this.settingsToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.settingsToolStripMenuItem.Text = Language.MainContextMenu.SettingsWord;
            this.settingsToolStripMenuItem.Click += new System.EventHandler(this.settingsToolStripMenuItem_Click);
            // 
            // iconReplacementModeToolStripMenuItem
            // 
            this.iconReplacementModeToolStripMenuItem.Name = "iconReplacementModeToolStripMenuItem";
            this.iconReplacementModeToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.iconReplacementModeToolStripMenuItem.Text = Language.General.IconReplacementMode;
            this.iconReplacementModeToolStripMenuItem.CheckOnClick = true;
            this.iconReplacementModeToolStripMenuItem.Checked = DockSettings.General.IconReplacementMode;
            this.iconReplacementModeToolStripMenuItem.Click += new System.EventHandler(this.iconReplacementModeToolStripMenuItem_Click);
            // 
            // pauseMouseHotkeysToolStripMenuItem
            // 
            this.pauseMouseHotkeysToolStripMenuItem.Name = "pauseMouseHotkeysToolStripMenuItem";
            this.pauseMouseHotkeysToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.pauseMouseHotkeysToolStripMenuItem.Text = Language.MainContextMenu.PauseMouseToggling;
            this.pauseMouseHotkeysToolStripMenuItem.CheckOnClick = true;
            this.pauseMouseHotkeysToolStripMenuItem.Click += new System.EventHandler(this.pauseMouseHotkeysToolStripMenuItem_Click);
            // 
            // OpenDataPointsDialog
            // 
            this.ChangeIconDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            this.ChangeIconDialog.Filter = "PNG|*.png";
            this.ChangeIconDialog.Multiselect = false;
            this.ChangeIconDialog.Title = Language.MainContextMenu.ChangeIcon;
            this.ChangeIconDialog.FileOk += new System.ComponentModel.CancelEventHandler(this.ChangeIconDialog_FileOk);
            // 
            // changeIconStripMenuItem
            // 
            this.changeIconToolStripMenuItem.Name = "changeIconToolStripMenuItem";
            this.changeIconToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.changeIconToolStripMenuItem.Text = Language.MainContextMenu.ChangeIcon;
            this.changeIconToolStripMenuItem.Click += new System.EventHandler(this.changeIconToolStripMenuItem_Click);
            // 
            // dockFolderToolStripMenuItem
            // 
            this.dockFolderToolStripMenuItem.Name = "dockFolderToolStripMenuItem";
            this.dockFolderToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.dockFolderToolStripMenuItem.Text = Language.MainContextMenu.DockFolder;
            this.dockFolderToolStripMenuItem.Click += new System.EventHandler(this.dockFolderToolStripMenuItem_Click);
            // 
            // blankIconToolStripMenuItem
            // 
            this.blankIconToolStripMenuItem.Name = "blankIconToolStripMenuItem";
            this.blankIconToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.blankIconToolStripMenuItem.Text = Language.MainContextMenu.blankIcon;
            this.blankIconToolStripMenuItem.Click += new System.EventHandler(this.blankIconToolStripMenuItem_Click);
            // 
            // stackFolderToolStripMenuItem
            // 
            this.stackFolderToolStripMenuItem.Name = "stackFolderToolStripMenuItem";
            this.stackFolderToolStripMenuItem.Size = new System.Drawing.Size(152, 22);
            this.stackFolderToolStripMenuItem.Text = "Folder Stack";
            this.stackFolderToolStripMenuItem.Click += new System.EventHandler(this.stackFolderToolStripMenuItem_Click);
            // 
            // 
            // TransparentObject
            // 
            this.ClientSize = new System.Drawing.Size(1, 1);

            this.MouseEnter += new EventHandler(CentreObject_MouseEnter);
            this.MouseDown += new MouseEventHandler(CentreObject_MouseDown);
            this.MouseUp += new MouseEventHandler(CentreObject_MouseUp);
            this.MouseMove += new MouseEventHandler(CentreObject_MouseMove);
            this.MouseWheel += new MouseEventHandler(CentreObject_MouseWheel);
            this.MouseHover += new EventHandler(CentreObject_MouseHover);

            this.DragEnter += new DragEventHandler(CentreObject_DragEnter);
            this.DragDrop += new DragEventHandler(CentreObject_DragDrop);

            this.KeyDown += new KeyEventHandler(CentreObject_KeyDown);
            this.ContextMenuStrip = contextMenuStrip;

            this.Deactivate += new EventHandler(CentreObject_Deactivate);

            //this.TopMost = true;
            this.Name = "TransparentObject";
            this.contextMenuStrip.ResumeLayout(false);
            this.ResumeLayout(false);
        }

        void CentreObject_Deactivate(object sender, EventArgs e)
        {
            ParentObject.itemDeactivated();
        }

        #endregion

        #region Variables

        public String SectionName;
        public int PreviousOpacity;
        Point previousMousePosition;

        /// <summary> 
        /// Use to store the difference between the global positon of the mouse cursor and the global position of an object.
        /// </summary>
        private Point StoredMouseOffset;
        #endregion

        #region Constructor

        /// <summary> 
        /// Create an alpha blended form with a transparent background that is movable and responds to mouse and key inputs.
        /// </summary>
        public CentreObject(MainForm TheParent, LanguageLoader.LanguageLoader LanguageData, SettingsLoader.SettingsLoader SettingsData, String Section, Size InitialSize)
        {
            Language = LanguageData;
            DockSettings = SettingsData;
            ParentObject = TheParent;
            ObjectSize = InitialSize;
            SectionName = Section;

            InitializeComponent();

            DoubleBuffered = true;
            this.AnimationTimer.Tick += new System.EventHandler(this.AnimationTimer_Tick);
            AnimationTimer.Interval = DockSettings.General.AnimationInterval;

            ShowInTaskbar = false;
            AllowDrop = true;

            LeftMouseButtonDown = false;
            ThisObjectMovedWithLeftMouse = false;
            StoredMouseOffset = new Point(0, 0);

            ObjectBitmap = new Bitmap(1, 1, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
            OverlayBitmap = new Bitmap(1, 1, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
            ObjectOpacity = DockSettings.CentreImage.DefaultOpacity;
            PreviousOpacity = ObjectOpacity;

            SetZLevel();

            SetBitmap();
        }

        #endregion

        public void SetZLevel()
        {
            if (DockSettings.General.zLevel == "Always On Bottom")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_BOTTOM, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else if (DockSettings.General.zLevel == "Normal")
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_NOTOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
            else
                Pinvoke.Win32.SetWindowPos(this.Handle, (IntPtr)Pinvoke.Win32.HWND_TOPMOST, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOMOVE | Pinvoke.Win32.SWP_NOSIZE
                    | Pinvoke.Win32.SWP_NOACTIVATE | Pinvoke.Win32.SWP_NOOWNERZORDER | Pinvoke.Win32.SWP_NOREDRAW | Pinvoke.Win32.SWP_NOSENDCHANGING);
        }

        private int TargetOpacity;
        private double OpacityRateOfChange;
        private double tempNewObjectOpacity;

        public void AnimateOpacity(int NewOpacity, int Duration)
        {
            if (Duration <= 0)
                return;

            TargetOpacity = NewOpacity;
            OpacityRateOfChange = ((double)NewOpacity - (double)ObjectOpacity) / ((double)Duration / (double)AnimationTimer.Interval);

            tempNewObjectOpacity = ObjectOpacity;
            AnimationTimer.Start();
        }

        private void AnimationTimer_Tick(object sender, EventArgs e)
        {
            tempNewObjectOpacity = tempNewObjectOpacity + OpacityRateOfChange;
            if ((OpacityRateOfChange > 0 && tempNewObjectOpacity > TargetOpacity) ||
                (OpacityRateOfChange < 0 && tempNewObjectOpacity < TargetOpacity))
                tempNewObjectOpacity = TargetOpacity;

            //DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, this.Location.X, this.Location.Y, false, 0, 0, 0, 0, true, (int)tempNewObjectOpacity);
            UpdateOpacity((int)tempNewObjectOpacity);

            if (ObjectOpacity == 0 || ObjectOpacity == 255 || (OpacityRateOfChange < 0 && ObjectOpacity <= TargetOpacity) ||
                (OpacityRateOfChange > 0 && ObjectOpacity >= TargetOpacity))
            {
                AnimationTimer.Stop();
                AnimationTimer.Enabled = false;
            }
        }

        public void UpdateAnimationTimerInterval(int NewInterval)
        {
            AnimationTimer.Interval = NewInterval;
        }

        public void SetBitmap()
        {
            Bitmap tempBitmap;
            try
            {
                if (DockSettings.CentreImage.Path.StartsWith(@".\"))
                {
                    tempBitmap = new Bitmap(Application.StartupPath + @DockSettings.CentreImage.Path.Substring(1, DockSettings.CentreImage.Path.Length - 1));
                }
                else
                {
                    tempBitmap = new Bitmap(@DockSettings.CentreImage.Path);
                }
            }
            catch (Exception)
            {
                tempBitmap = new Bitmap(ImageResources.CircleDockIconCentreImage);
            }

            SetBitmapManaged(ref tempBitmap);
        }

        #region Object Events

        /// <summary> 
        /// Move the text label when this object is moved.
        /// </summary>
        protected override void OnMove(EventArgs e)
        {
            base.OnMove(e);
        }

        void CentreObject_MouseHover(object sender, EventArgs e)
        {
            this.Activate();
        }

        void CentreObject_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Left)
                ParentObject.RotateDock(-1.0);
            else if (e.KeyCode == Keys.Right)
                ParentObject.RotateDock(1.0);
            else if (e.KeyCode == Keys.Down)
                ParentObject.RotateDock(-1.0);
            else if (e.KeyCode == Keys.Up)
                ParentObject.RotateDock(1.0);
        }

        /// <summary> 
        /// Add files to the dock using Drag and Drop
        /// </summary>
        void CentreObject_DragDrop(object sender, DragEventArgs e)
        {
            //string[] files2 = e.Data.GetData(System.Windows.Forms.DataFormats.StringFormat, true) as string[];

            //try
            //{

            //    Console.WriteLine(files2[0]);
            //}
            //catch (Exception)
            //{ }

            string[] files = e.Data.GetData(DataFormats.FileDrop) as string[];
            if (DockSettings.General.IconReplacementMode && files != null && files.Length == 1 && files[0].ToUpper().EndsWith(@".PNG"))
            {
                ChangeIconDialog.FileName = files[0];
                ChangeIconDialog_FileOk(this, null);
            }
            else if (files != null)
            {
                ParentObject.CentreObject.Activate();
                for (int i = 0; i < files.Length; i++)
                {
                    ParentObject.AddDockItem("[link]", files[i], "Link", "", "");
                    //this.Activate();
                }

                ParentObject.PositionCurrentLevel();
            }
        }

        /// <summary> 
        /// Change the cursor when using Drag and Drop
        /// </summary>
        void CentreObject_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Link;
        }

        void CentreObject_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta < 0)
                ParentObject.RotateDock(1.0);
            else
                ParentObject.RotateDock(-1.0);
        }

        void CentreObject_MouseMove(object sender, MouseEventArgs e)
        {
            if (DockSettings.General.lockDockPosition == false && LeftMouseButtonDown)
            {
                if (ThisObjectMovedWithLeftMouse == true || Math.Abs(Cursor.Position.X - previousMousePosition.X) > DockSettings.General.ClickSensitivity
                    || Math.Abs(Cursor.Position.Y - previousMousePosition.Y) > DockSettings.General.ClickSensitivity)
                {
                    ThisObjectMovedWithLeftMouse = true;

                    this.Location = new Point(Cursor.Position.X - StoredMouseOffset.X, Cursor.Position.Y - StoredMouseOffset.Y);

                    if (ParentObject != null)
                    {
                        ParentObject.ChildMoved(this);
                    }
                }
            }
        }

        void CentreObject_MouseUp(object sender, MouseEventArgs e)
        {
            if (ThisObjectMovedWithLeftMouse == false)
            {
                if (e.Button == MouseButtons.Left && LeftMouseButtonDown)
                {
                    LeftMouseButtonDown = false;
                    if (ParentObject.DockIsVisible || DockSettings.General.zLevel == @"Always On Bottom")
                    {
                        //DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, PreviousOpacity);
                        DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, DockSettings.DockItems.DefaultOpacity);
                        PreviousOpacity = ObjectOpacity;
                    }
                    else
                    {
                        return;
                    }

                    if (ParentObject.AtRootLevel())
                    {
                        if (ParentObject.showingLevel != true)
                        {
                            if (DockSettings.CentreImage.ShowStartMenuWhenClicked)
                            {

                                IntPtr StartMenuHandle = (IntPtr)Pinvoke.Win32.FindWindow("DV2ControlHost", null);
                                IntPtr taskBarWnd = (IntPtr)Pinvoke.Win32.FindWindow("Shell_TrayWnd", null);
                                IntPtr startWnd = Pinvoke.Win32.FindWindowEx(taskBarWnd, IntPtr.Zero, "Button", "Start");

                                Pinvoke.Win32.WINDOWINFO StartMenuInfo = new Pinvoke.Win32.WINDOWINFO();
                                Pinvoke.Win32.GetWindowInfo(StartMenuHandle, ref StartMenuInfo);

                                Point StartMenuBestPos = new Point(this.Location.X, this.Location.Y - (StartMenuInfo.rcWindow.bottom - StartMenuInfo.rcWindow.top));

                                if (StartMenuBestPos.Y < SystemInformation.VirtualScreen.Top)
                                    StartMenuBestPos = new Point(StartMenuBestPos.X, SystemInformation.VirtualScreen.Top);

                                if (StartMenuBestPos.Y + (StartMenuInfo.rcWindow.bottom - StartMenuInfo.rcWindow.top) > SystemInformation.VirtualScreen.Bottom)
                                    StartMenuBestPos = new Point(StartMenuBestPos.X, SystemInformation.VirtualScreen.Bottom - (StartMenuInfo.rcWindow.bottom - StartMenuInfo.rcWindow.top));

                                if (StartMenuBestPos.X + (StartMenuInfo.rcWindow.right - StartMenuInfo.rcWindow.left) > SystemInformation.VirtualScreen.Right)
                                    StartMenuBestPos = new Point(SystemInformation.VirtualScreen.Right - (StartMenuInfo.rcWindow.right - StartMenuInfo.rcWindow.left), StartMenuBestPos.Y);

                                if (StartMenuBestPos.X < SystemInformation.VirtualScreen.Left)
                                    StartMenuBestPos = new Point(SystemInformation.VirtualScreen.Left, StartMenuBestPos.Y);

                                Pinvoke.Win32.SetWindowPos(taskBarWnd, IntPtr.Zero, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOSIZE | Pinvoke.Win32.SWP_SHOWWINDOW |
                                    Pinvoke.Win32.SWP_NOMOVE);
                                Pinvoke.Win32.SetWindowPos(startWnd, IntPtr.Zero, 0, 0, 0, 0, Pinvoke.Win32.SWP_NOSIZE | Pinvoke.Win32.SWP_SHOWWINDOW |
                                    Pinvoke.Win32.SWP_NOMOVE);

                                //long classLong = (long)Pinvoke.Win32.GetClassLongPtr(StartMenuHandle, Constants.GCLConstants.GCL_STYLE);
                                //classLong = (classLong & ~(long)Constants.ClassStyles.CS_DROPSHADOW) | (long)Constants.ClassStyles.CS_OWNDC;

                                //Pinvoke.Win32.SetClassLong(StartMenuHandle, Constants.GCLConstants.GCL_STYLE, (IntPtr)classLong);
                                //Pinvoke.Win32.SendMessage(StartMenuHandle, Constants.WindowMessageConstants.WM_PAINT, 0, 0);
                                //Pinvoke.Win32.RedrawWindow(StartMenuHandle, IntPtr.Zero, IntPtr.Zero,
                                //    0x0100 | 0x0001 | 0x0040 | 0x0004);


                                Pinvoke.Win32.SetWindowPos(StartMenuHandle, (IntPtr)Pinvoke.Win32.HWND_TOPMOST, StartMenuBestPos.X, StartMenuBestPos.Y, 0, 0, Pinvoke.Win32.SWP_NOSIZE |
                                    Pinvoke.Win32.SWP_SHOWWINDOW | Pinvoke.Win32.SWP_FRAMECHANGED);

                                //Thread.Sleep(5000);
                                //Pinvoke.Win32.SetWindowPos(StartMenuHandle, (IntPtr)Pinvoke.Win32.HWND_TOPMOST, StartMenuBestPos.X, StartMenuBestPos.Y, 0, 0, Pinvoke.Win32.SWP_NOSIZE |
                                //    Pinvoke.Win32.SWP_HIDEWINDOW | Pinvoke.Win32.SWP_FRAMECHANGED);

                                //long classLong = (long)Pinvoke.Win32.GetClassLongPtr(StartMenuHandle, Constants.GCLConstants.GCL_STYLE);
                                //classLong = (classLong & ~(long)Constants.ClassStyles.CS_DROPSHADOW) | (long)Constants.ClassStyles.CS_OWNDC;

                                //Pinvoke.Win32.SetClassLong(StartMenuHandle, Constants.GCLConstants.GCL_STYLE, (IntPtr)classLong);
                                //Pinvoke.Win32.SendMessage(StartMenuHandle, Constants.WindowMessageConstants.WM_PAINT, 0, 0);
                                //Pinvoke.Win32.RedrawWindow(StartMenuHandle, IntPtr.Zero, IntPtr.Zero,
                                //    0x0100 | 0x0001 | 0x0040 | 0x0004);

                                //Pinvoke.Win32.SetForegroundWindow(StartMenuHandle);

                                //StartMenuContainer testmenu = new StartMenuContainer();
                                //testmenu.TopLevel = true;
                                //Pinvoke.Win32.SetParent(testmenu.Handle, (IntPtr)Pinvoke.Win32.GetDesktopWindow());
                            }

                            if (DockSettings.DockItems.HideDockAfterExecution)
                                ParentObject.HideAll();
                        }
                    }
                    else
                    {
                        ParentObject.ShowLevelUp();
                    }
                }
            }
            else
            {
                ThisObjectMovedWithLeftMouse = false;

                if (e.Button == MouseButtons.Left && LeftMouseButtonDown)
                {
                    LeftMouseButtonDown = false;
                    if (ParentObject.DockIsVisible || DockSettings.General.zLevel == @"Always On Bottom")
                    {
                        //DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, PreviousOpacity);
                        DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, DockSettings.DockItems.DefaultOpacity);
                        PreviousOpacity = ObjectOpacity;
                    }
                }
            }
        }

        void CentreObject_MouseDown(object sender, MouseEventArgs e)
        {
            this.Activate();
            this.BringToFront();
            if (e.Button == MouseButtons.Left)
            {
                LeftMouseButtonDown = true;
                ThisObjectMovedWithLeftMouse = false;
                PreviousOpacity = ObjectOpacity;
                DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, true, ObjectOpacity / 2);
                StoredMouseOffset = new Point(Cursor.Position.X - this.Location.X, Cursor.Position.Y - this.Location.Y);
                previousMousePosition = Cursor.Position;
            }
        }


        void CentreObject_MouseEnter(object sender, EventArgs e)
        {
            this.BringToFront();
            this.ParentObject.BackgroundObject.DockCanHide = false;
            //this.ParentObject.BackgroundObject.dwellTimer.Stop();
        }

        #endregion

        #region Context Menu

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.ChildRequestAction("QUIT");
        }

        private void dockFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.CentreObject.Activate();
            ParentObject.AddDockItem("[dockfolder]", "", Language.MainContextMenu.DockFolder, DockSettings.Folders.DockFolderImagePath, Language.MainContextMenu.DockFolder);
            ParentObject.PositionCurrentLevel();
        }

        private void blankIconToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.CentreObject.Activate();
            ParentObject.AddDockItem("[blanklink]", "", "Link", "", "");
            ParentObject.PositionCurrentLevel();
        }
        //
        // Add a Folder Stack to the Dock
        //
        private void stackFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.CentreObject.Activate();
            ParentObject.AddDockItem("[stack]", ".\\System\\Miscellaneous\\StandaloneStack.Exe", "Stack", "", "Change Me Now!");
            ParentObject.PositionCurrentLevel();
        }

        private void toggleAutoHide_Click(object sender, EventArgs e)
        {
            DockSettings.Toggling.AutoHide = toggleAutoHide.Checked;
        }

        private void hideToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.HideAll();
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.ShowSettingsPanel();
        }

        private void changeIconToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeIconDialog.InitialDirectory = Application.StartupPath + @"\System\Icons";
            ChangeIconDialog.ShowDialog();
        }

        private void iconReplacementModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DockSettings.General.IconReplacementMode = iconReplacementModeToolStripMenuItem.Checked;
            DockSettings.SetEntry("General", "IconReplacementMode", iconReplacementModeToolStripMenuItem.Checked.ToString());

            if (DockSettings.General.IconReplacementMode)
            {
                FileOps.FileOps FileOperations = new FileOps.FileOps(IntPtr.Zero, Language, DockSettings);
                FileOperations.Open(Application.StartupPath + @"\System\Icons", @"", System.Diagnostics.ProcessWindowStyle.Normal, this.Handle);
            }
        }

        private void pauseMouseHotkeysToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ParentObject.pauseMouseHotkeysToolStripMenuItem.Checked = pauseMouseHotkeysToolStripMenuItem.Checked;
            ParentObject.SetMouseKeys();
        }

        private void contextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            iconReplacementModeToolStripMenuItem.Checked = DockSettings.General.IconReplacementMode;
            pauseMouseHotkeysToolStripMenuItem.Checked = ParentObject.pauseMouseHotkeysToolStripMenuItem.Checked;
        }

        private void contextMenuStrip_MouseEnter(object sender, EventArgs e)
        {
            this.ParentObject.BackgroundObject.DockCanHide = false;
        }

        private void ChangeIconDialog_FileOk(object sender, CancelEventArgs e)
        {
            String NewImagePath;
            if (ChangeIconDialog.FileName.StartsWith(Application.StartupPath + @"\"))
            {
                NewImagePath = @"." + ChangeIconDialog.FileName.Substring(Application.StartupPath.Length, ChangeIconDialog.FileName.Length - Application.StartupPath.Length);
            }
            else
            {
                NewImagePath = ChangeIconDialog.FileName;
            }

            DockSettings.SetEntry("CentreImage", "Path", NewImagePath);
            DockSettings.CentreImage.Path = NewImagePath;

            SetBitmap();
            Size ObjectSize = this.ObjectSize;
            DrawBitmapManaged(ObjectSize.Width, ObjectSize.Height, false, 0, 0, false, 0, 0, 0, 0, false, 0);
        }

        #endregion
    }
}
